'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
}

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

var BABEL_LOADER_PATH = 'next/dist/build/babel/loader';

var MISSING_NEXT_CONFIG_ERROR = new Error('Expected Webpack rule to have `nextConfig` option.');

function filterByArray(value) {
    return Array.isArray(value);
}

function filterByRecord(value) {
    return typeof value === 'object' && value !== null;
}

function filterByNextConfig(value) {
    return filterByRecord(value) && typeof value.distDir === 'string';
}

function filterByOneOf(rule) {
    return (Object.prototype.hasOwnProperty.call(rule, 'oneOf') &&
        Array.isArray(rule.oneOf));
}

var BABEL_LOADER_REGEXP = /next[/\\]dist[/\\]build[/\\]babel[/\\]loader/;

function findBabelRule(rule) {
    return (typeof rule.use === 'object' &&
        !Array.isArray(rule.use) &&
        typeof rule.use.loader === 'string' &&
        BABEL_LOADER_REGEXP.test(rule.use.loader));
}

function findNextSwcRule(rule) {
    return (typeof rule.use === 'object' &&
        !Array.isArray(rule.use) &&
        typeof rule.use.loader === 'string' &&
        rule.use.loader === 'next-swc-loader' &&
        typeof rule.use.options === 'object');
}

var ALL_FILES_REGEXP = /.*/;

function mapBabelOptionsToRule(babelOptions) {
    return __assign({ test: ALL_FILES_REGEXP }, babelOptions);
}

function mapBabelOptionsToRuleOverrides(babelOptions) {
    return [mapBabelOptionsToRule(babelOptions)];
}

var BabelWebpackConfig = (function () {
    function BabelWebpackConfig(webpackConfig, webpackConfigContext, babelOptions) {
        this._overrides = mapBabelOptionsToRuleOverrides(babelOptions);
        this._webpackConfig = webpackConfig;
        this._webpackConfigContext = webpackConfigContext;
    }
    Object.defineProperty(BabelWebpackConfig.prototype, "_moduleOptions", {
        get: function () {
            var moduleOptions = this._webpackConfig.module;
            if (typeof moduleOptions === 'undefined') {
                return;
            }
            return this._mapModuleOptionsToBabelModuleOptions(moduleOptions);
        },
        enumerable: false,
        configurable: true
    });
    BabelWebpackConfig.prototype.toJSON = function () {
        var moduleOptions = this._moduleOptions;
        if (typeof moduleOptions === 'undefined') {
            return this._webpackConfig;
        }
        return __assign(__assign({}, this._webpackConfig), { module: moduleOptions });
    };
    BabelWebpackConfig.prototype._mapModuleOptionsToBabelModuleOptions = function (moduleOptions) {
        var rules = moduleOptions.rules;
        if (typeof rules === 'undefined') {
            return moduleOptions;
        }
        return __assign(__assign({}, moduleOptions), { rules: this._mapRulesToBabelRules(rules) });
    };
    BabelWebpackConfig.prototype._mapNextSwcLoaderOptionsToBabelLoaderOptions = function (options) {
        var hasReactRefresh = options.hasReactRefresh, isServer = options.isServer, nextConfig = options.nextConfig, pagesDir = options.pagesDir;
        if (!filterByNextConfig(nextConfig)) {
            throw MISSING_NEXT_CONFIG_ERROR;
        }
        return {
            cwd: this._webpackConfigContext.dir,
            development: this._webpackConfigContext.dev,
            distDir: "".concat(this._webpackConfigContext.dir, "/").concat(nextConfig.distDir),
            hasJsxRuntime: true,
            hasReactRefresh: hasReactRefresh,
            isServer: isServer,
            overrides: this._overrides,
            pagesDir: pagesDir,
        };
    };
    BabelWebpackConfig.prototype._mapNextSwcLoaderToBabelLoader = function (_a) {
        var options = _a.options;
        return {
            loader: BABEL_LOADER_PATH,
            options: this._mapNextSwcLoaderOptionsToBabelLoaderOptions(options),
        };
    };
    BabelWebpackConfig.prototype._mapNextSwcRuleToBabelRule = function (rule) {
        return __assign(__assign({}, rule), { use: this._mapNextSwcLoaderToBabelLoader(rule.use) });
    };
    BabelWebpackConfig.prototype._mapOneOfRuleToBabelOneOfRule = function (rule) {
        var mapRuleToBabelRule = this._mapRuleToBabelRule.bind(this);
        return __assign(__assign({}, rule), { oneOf: rule.oneOf.map(mapRuleToBabelRule) });
    };
    BabelWebpackConfig.prototype._mapRuleToBabelRule = function (rule) {
        if (rule === '...') {
            return '...';
        }
        if (filterByOneOf(rule)) {
            return this._mapOneOfRuleToBabelOneOfRule(rule);
        }
        if (findBabelRule(rule)) {
            return __assign(__assign({}, rule), { use: __assign(__assign({}, rule.use), { options: this._overrideBabelLoaderOptions(rule.use.options) }) });
        }
        if (findNextSwcRule(rule)) {
            return this._mapNextSwcRuleToBabelRule(rule);
        }
        return rule;
    };
    BabelWebpackConfig.prototype._mapRulesToBabelRules = function (rules) {
        var mapRuleToBabelRule = this._mapRuleToBabelRule.bind(this);
        return rules.map(mapRuleToBabelRule);
    };
    BabelWebpackConfig.prototype._mergeOverrides = function (overrides) {
        if (!filterByArray(overrides)) {
            return this._overrides;
        }
        return __spreadArray(__spreadArray([], __read(overrides), false), __read(this._overrides), false);
    };
    BabelWebpackConfig.prototype._overrideBabelLoaderOptions = function (options) {
        if (typeof options !== 'object') {
            return {
                overrides: this._overrides,
            };
        }
        return __assign(__assign({}, options), { overrides: this._mergeOverrides(options.overrides) });
    };
    return BabelWebpackConfig;
}());

function generateNextBabelWebpackConfig(nextBabelConfig, webpackConfig, webpackConfigContext) {
    var babelOptions = nextBabelConfig(webpackConfigContext);
    if (typeof babelOptions === 'undefined') {
        return webpackConfig;
    }
    return new BabelWebpackConfig(webpackConfig, webpackConfigContext, babelOptions).toJSON();
}

function configureNextBabel(nextBabelConfig) {
    return function webpack(webpackConfig, webpackConfigContext) {
        return generateNextBabelWebpackConfig(nextBabelConfig, webpackConfig, webpackConfigContext);
    };
}

exports["default"] = configureNextBabel;
